/*
 * Copyright Justin Armstrong 2004. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#ifndef WEATHERAPPUI_H
#define WEATHERAPPUI_H

// INCLUDES
#include <eikapp.h>
#include <eikdoc.h>
#include <e32std.h>
#include <coeccntx.h>
#include <aknappui.h>
#include <aknlists.h>
#include <eikclb.h>
#include <aknlistquerydialog.h> 
#include <aknquerydialog.h>
#include <aknwaitdialog.h>  
#include <aknnavi.h> 
#include <aknnavide.h>
#include <akntitle.h> 
#include <akncontext.h> 

#include "weatherparser.h"
#include "constants.h"
#include "usersettingsdialog.h"
#include "locationform.h"
#include "location.h"
#include "thermometer.h"
#include "SocketsEngine.h"
#include "uinotify.h"

#ifndef WEATHER_MBG
#define WEATHER_MBG
#include <weather.mbg>
#endif



class CWeatherContainer;

class CLocation;


struct TUserSettings
    {
    
    /*TTemperatureUnit*/TInt iTempUnits;
    /*TSpeedUnit*/TInt iSpeedUnits;
    TInt iDistanceUnits;
    TBool iDetailedCloudReports;
    TBool iUseProxy;
    TInetAddr iProxyIpAddress;
    TInt iProxyPort;
    };


struct CGlobals
    {

    ~CGlobals()
        {
        delete iParser;
        }

    CWeatherParser* iParser;
    TUserSettings  iSettings;
    };



// copied from Symbian Knowledgebase FAQ-0535
class TKeyArrayPtr : public TKeyArrayFix
{
    public:
    
    inline TKeyArrayPtr(TInt aOffset, TKeyCmpText aType) :TKeyArrayFix(aOffset, aType) {}
    inline TKeyArrayPtr(TInt aOffset, TKeyCmpText aType, TInt aLength) :TKeyArrayFix(aOffset, aType, aLength) {}
    inline TKeyArrayPtr(TInt aOffset, TKeyCmpNumeric aType) :TKeyArrayFix(aOffset, aType) {}
    virtual TAny* At(TInt aIndex) const;
};






#define GLOBALS ((CGlobals*)Dll::Tls())

class CWeatherAppUi : public CAknAppUi, MHttpNotify, MProgressDialogCallback
    {
    public: // // Constructors and destructor

        /**
        * EPOC default constructor.
        */      
        void ConstructL();

        /**
        * Destructor.
        */      
        ~CWeatherAppUi();
        
        void EditLocationsCmdL();

        TBool ShowLocationsPopupListL(const TDesC& aTitle, TInt& aSelectedIndex);


        void CurrentLocationCmdL(TInt aIndex);

        void DisplayLocation();

        void SaveLocationsL();
        void LoadLocationsL();

        void SaveUserSettingsL();
        void LoadUserSettingsL();

        void SocketStatusNotify(const TDesC& aMessage, TBool aTerminal);
        void SocketStatusNotify(TInt aMsgResourceId, TBool aTerminal);

        void DialogDismissedL(TInt aButtonId);


        void HttpLoadedNotify(TPtrC aBody, TInt aResponseCode);

    private:
        // From MEikMenuObserver
        void DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane);

    private:
        /**
        * From CEikAppUi, takes care of command handling.
        * @param aCommand command to be handled
        */
        void HandleCommandL(TInt aCommand);

        /**
        * From CEikAppUi, handles key events.
        * @param aKeyEvent Event to handled.
        * @param aType Type of the key event. 
        * @return Response code (EKeyWasConsumed, EKeyWasNotConsumed). 
        */
        virtual TKeyResponse HandleKeyEventL(
            const TKeyEvent& aKeyEvent,TEventCode aType);

    private:

        HBufC* iTxtFormatDelete;
        HBufC* iTxtDeleteLocation;
        HBufC* iTxtModifyLocation;


        CWeatherContainer* iAppContainer; 

        CAknWaitDialog* iWaitDialog;

        CAknNavigationControlContainer* iNaviContainer;
        CAknNavigationDecorator* iTimeLabel;

        CSocketsEngine* iSocketsEngine;

        CArrayPtrFlat<CLocation>* iLocationArray;

        CLocation* iCurrentLocation;
        TInt iLastLocationIndex;

    };

#endif

// End of File
