
/*
 * Copyright Justin Armstrong 2004. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


#ifndef WEATHERVIEW_H
#define WEATHERVIEW_H

// INCLUDES
#include <coecntrl.h>
#include <coeccntx.h>
#include <aknappui.h>
#include <eiksbfrm.h>
#include <eiksbobs.h>
#include <eikapp.h>
#include "stringintdict.h"
#include "bitmaplabel.h"
#include "weatherinfo.h"
#include "location.h"
#include "weatherutils.h"

class CEikLabel;


class CWeatherView : public CCoeControl
    {
    public:
        
        CWeatherView();
        
        /**
        * EPOC default constructor.
        * @param aRect Frame rectangle for container.
        */
        void ConstructL(const TRect& aRect);

        /**
        * Destructor.
        */
        ~CWeatherView();

    public: // New functions

        void DisplayLocation(CLocation* aLocation);

    public: // Functions from base classes


    private: // Functions from base classes

       /**
        * From CoeControl,SizeChanged.
        */
        void SizeChanged();

       /**
        * From CoeControl,CountComponentControls.
        */
        TInt CountComponentControls() const;

       /**
        * From CCoeControl,ComponentControl.
        */
        CCoeControl* ComponentControl(TInt aIndex) const;

       /**
        * From CCoeControl,Draw.
        */
        void Draw(const TRect& aRect) const;



    private:

        void AddLabelL(const TDesC& aString, TInt aYPos);
        void AddConditionLabelL(TWeatherCondition&, TInt aYPos);
        void AddCloudLabelL(TCloud& cloud, TInt aYPos);
        void AddCloudSummaryLabelL(CArrayFixFlat<TCloud>* aClouds, TInt aYPos);

    private: //data
        

        TRect iMaxRect;


        const CFont* iDenseFont;
        CArrayPtrFlat<CCoeControl>* iControlArray;

        CStringIntDict* iPhenomenonDict;
        CStringIntDict* iModifierDict;
        CStringIntDict* iIntensityDict;

        CStringIntDict* iCloudDict;

        CStringIntDict* iCompassDict;

    };

#endif

// End of File
