
/*
 * Copyright Justin Armstrong 2004. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


#include "bitmaplabel.h"

CBitmapLabel::CBitmapLabel()
    {

    }

CBitmapLabel::~CBitmapLabel()
    {

    delete iBitmap;
    delete iLabel;
    }


void CBitmapLabel::SetContainerWindowL(const CCoeControl& aContainer)
    {
    CCoeControl::SetContainerWindowL(aContainer);
    iLabel->SetContainerWindowL(aContainer);

    }

void CBitmapLabel::ConstructL(const CFont* aFont, TDesC& aString, TInt aBitmapId)
    {
    
    TFileName app = CEikonEnv::Static()->EikAppUi()->Application()->AppFullName();
    TParsePtr parser(app);
    TFileName mbmFile = parser.Drive();
    mbmFile.Append(_L("\\system\\apps\\weather\\Weather.mbm"));

    iString = aString;
    iFont = aFont;

    iLabel = new (ELeave) CEikLabel;
    iLabel->SetFont(aFont);
    iLabel->SetTextL( aString );

    if (aBitmapId > 0)
        {
        iBitmap = new (ELeave) CFbsBitmap();
        User::LeaveIfError(iBitmap->Load(mbmFile, aBitmapId));
        }

    ActivateL();
    
    }

CCoeControl* CBitmapLabel::ComponentControl(TInt /*aIndex*/) const
    {

    return iLabel;

    }


TInt CBitmapLabel::CountComponentControls() const
    {

    return 1;

    }

void CBitmapLabel::Draw(const TRect& aRect) const
    {



    RDebug::Print(_L("CBitmapLabel::Draw"));
    CWindowGc& gc = SystemGc();


    gc.SetPenStyle(CGraphicsContext::ENullPen);
    gc.SetBrushColor(KRgbWhite);
    gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
    gc.DrawRect(aRect);


    gc.SetPenColor(KRgbBlack);
    gc.SetPenStyle(CGraphicsContext::ESolidPen);


    if (iBitmap)
        {
        TRect bitmapRect(aRect.iTl, TSize(30, 20));
        gc.DrawBitmap(bitmapRect, iBitmap);
        }



    TRect textRect = aRect;
    textRect.iTl.iX += 34;
    textRect.iBr.iY += 20;

    iLabel->SetExtent(textRect.iTl, textRect.Size());  

/*
    gc.UseFont(iFont);

    gc.DrawText(iString, textRect, iFont->AscentInPixels(), CGraphicsContext::ELeft);

    gc.DiscardFont();
*/
    }


void CBitmapLabel::SizeChanged()
    {

    DrawNow();

    }
