
/*
 * Copyright Justin Armstrong 2004. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

#include "stringintdict.h"



CStringIntTuple::CStringIntTuple()
    {

    }

CStringIntTuple::~CStringIntTuple()
    {
    if (iString) delete iString;
    }


void CStringIntTuple::ConstructL(TResourceReader& aReader)
    {

    iInt = aReader.ReadInt16();
    iString = aReader.ReadHBufCL();


    RDebug::Print(_L("CStringIntTuple::ConstructL iString=%S iInt=%d"), 
                    iString, iInt);

    }

CStringIntTuple* CStringIntTuple::NewLC(TResourceReader& aReader)
	{
	CStringIntTuple* self=new (ELeave) CStringIntTuple;
	CleanupStack::PushL(self);
	self->ConstructL(aReader);
	return self;
	}


//////////////////////////////////////////////////////////////////////
//
//////////////////////////////////////////////////////////////////////

CStringIntDict::CStringIntDict()
    {

    }

CStringIntDict::~CStringIntDict()
    {
    iStringIntArray->ResetAndDestroy();
    delete iStringIntArray;
    }



//static const TKeyArrayFix _strPairKey(_FOFF(CStringIntTuple,iName), ECmpFolded);
//#define strPairKey ((TKeyArrayFix)_strPairKey)

void CStringIntDict::ConstructL(TResourceReader& aReader)
	{

    TInt size = aReader.ReadInt16();


	iStringIntArray = new (ELeave) CArrayPtrFlat<CStringIntTuple> (size); 

    TInt i;
	for (i = 0; i < size ; i++)
		{
        CStringIntTuple* tuple = CStringIntTuple::NewLC(aReader);
		iStringIntArray->AppendL(tuple);

		CleanupStack::Pop();
		}



    for (i = 0; i < size ; i++)
        {
        CStringIntTuple* tuple = iStringIntArray->At(i);
        RDebug::Print(_L("A %d <%S>--><%d>"), i, tuple->iString, tuple->iInt);

        }

	}

CStringIntDict* CStringIntDict::NewLC(TResourceReader& aReader)
	{
	CStringIntDict* self = new (ELeave) CStringIntDict;
	CleanupStack::PushL(self);
	self->ConstructL(aReader);

	return self;
	}

TInt CStringIntDict::GetIntForString(TPtrC aName)
    {

    TInt size = iStringIntArray->Count();

    TInt i;
    for (i = 0; i < size; i++)
        {
        CStringIntTuple* tuple = iStringIntArray->At(i);
        if (aName.Compare(*(tuple->iString)) == 0)
            return tuple->iInt;

        }

    return ETupleNotFound;

    }


HBufC* CStringIntDict::GetStringForInt(TInt aInt)
    {
    
    TInt size = iStringIntArray->Count();

    TInt i;
    for (i = 0; i < size; i++)
        {
        CStringIntTuple* tuple = iStringIntArray->At(i);
        if (tuple->iInt == aInt)
            return tuple->iString;

        }

    return NULL;

    }