/*
 * Copyright Justin Armstrong 2004. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


#include "thermometer.h"

#include "weatherutils.h"


#define THERM_HEIGHT 30 //pixels
#define THERM_MIN_TEMP -10  //celsius
#define THERM_PIXELS_PER_DEGREE 2
#define THERM_MAX_TEMP (THERM_MIN_TEMP+(THERM_HEIGHT*THERM_PIXELS_PER_DEGREE))


void _DrawTemperatureOnBitmap(TInt aTemperature, CGraphicsContext* bitmapContext, CFbsBitmapDevice* bitmapDevice)
    {

    TRect textRect(TPoint(4, 2), TSize(36, 20));

    CFont* font;
    _LIT(KTxtFontName,"LatinBold12");
    TFontSpec fontSpec(KTxtFontName(),textRect.Height());
    bitmapDevice->GetNearestFontInPixels(font, fontSpec);
    bitmapContext->UseFont(font);
    bitmapContext->SetPenColor(KRgbBlack);
    bitmapContext->SetPenStyle(CGraphicsContext::ESolidPen);


    TBuf<6> tempString;
    MakeTempString(tempString, aTemperature);

    bitmapContext->DrawText(tempString, textRect, font->AscentInPixels()+6, CGraphicsContext::ERight);

    bitmapContext->DiscardFont();


    }

void CThermometer::DrawTemperatureL(TInt aTemperature, CFbsBitmap*& aBitmap, CFbsBitmap*& aMaskBitmap)
    {

    TFileName app = CEikonEnv::Static()->EikAppUi()->Application()->AppFullName();
    TParsePtr parser(app);
    TFileName mbmFile = parser.Drive();
    mbmFile.Append(_L("\\system\\apps\\weather\\Weather.mbm"));


    aBitmap = new (ELeave) CFbsBitmap();
    CleanupStack::PushL(aBitmap);
    User::LeaveIfError(aBitmap->Load(mbmFile, EMbmWeatherTherm));

    aMaskBitmap = new (ELeave) CFbsBitmap();
    CleanupStack::PushL(aMaskBitmap);
    User::LeaveIfError(aMaskBitmap->Load(mbmFile, EMbmWeatherTherm_mask));

    RDebug::Print(_L("seems to have loaded okay"));


    CGraphicsContext* bitmapContext=NULL;
    CFbsBitmapDevice* bitmapDevice = CFbsBitmapDevice::NewL(aBitmap);
    CleanupStack::PushL(bitmapDevice);
    User::LeaveIfError(bitmapDevice->CreateContext(bitmapContext));
    CleanupStack::PushL(bitmapContext);


    CGraphicsContext* maskContext=NULL;
    CFbsBitmapDevice* maskDevice = CFbsBitmapDevice::NewL(aMaskBitmap);
    CleanupStack::PushL(maskDevice);
    User::LeaveIfError(maskDevice->CreateContext(maskContext));
    CleanupStack::PushL(maskContext);


    //draw the temp, in both the bitmap and the mask


    
    _DrawTemperatureOnBitmap(aTemperature, bitmapContext, bitmapDevice);
    _DrawTemperatureOnBitmap(aTemperature, maskContext, maskDevice);



    //draw the mercury

    TInt mercuryHeight;

    if (aTemperature < THERM_MIN_TEMP)
        mercuryHeight = 0;
    else if (aTemperature > THERM_MAX_TEMP)
        mercuryHeight = THERM_HEIGHT;
    else mercuryHeight = (aTemperature - THERM_MIN_TEMP)/THERM_PIXELS_PER_DEGREE;

    TPoint fromPt(5,31);
    TPoint endPt(5,31-mercuryHeight);
    bitmapContext->SetPenColor(KRgbRed);
    bitmapContext->DrawLine(fromPt, endPt);

    CleanupStack::PopAndDestroy();
    CleanupStack::PopAndDestroy();

    CleanupStack::PopAndDestroy();
    CleanupStack::PopAndDestroy();

    CleanupStack::Pop();
    CleanupStack::Pop();

    }

