/*
 * Copyright Justin Armstrong 2004. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

// INCLUDE FILES
#include "weatherappui.h"
#include "weathercontainer.h" 
#include <weather.rsg>
#include "weather.hrh"
#include "utf.h"

#include "aboutdialog.h"

#include <avkon.hrh>
#include <aknnotewrappers.h>

#include <intconninit.h> 
#include <commdb.h>
#include <apaccesspointitem.h> 
#include <apdatahandler.h> 
#include <apselect.h> 




// copied from Symbian Knowledgebase FAQ-0535
TAny* TKeyArrayPtr::At(TInt aIndex) const
{
    if (aIndex==KIndexPtr)
        return *(TUint8**)iPtr+iKeyOffset;

    return *(TUint8**)iBase->Ptr(aIndex*sizeof(TUint8**)).Ptr()+iKeyOffset;
}




//---------------------------------



_LIT(KEmptyLocations, "location array empty");

void CWeatherAppUi::ConstructL()
    {
    BaseConstructL();
    iAppContainer = new (ELeave) CWeatherContainer();
    iAppContainer->SetMopParent(this);
    iAppContainer->ConstructL( ClientRect() );
    AddToStackL( iAppContainer );


    CGlobals* globals = new (ELeave) CGlobals;
    Dll::SetTls(globals);

    GLOBALS->iParser = CWeatherParser::NewLC(iCoeEnv);

    iTxtFormatDelete = iCoeEnv->AllocReadResourceL(R_TEXT_FORMAT_DELETE);
    iTxtDeleteLocation = iCoeEnv->AllocReadResourceL(R_TEXT_DELETE_LOCATION);
    iTxtModifyLocation = iCoeEnv->AllocReadResourceL(R_TEXT_MODIFY_LOCATION);

    iSocketsEngine = CSocketsEngine::NewL(*this);

    LoadLocationsL();

    LoadUserSettingsL();

    DisplayLocation(); 


    CleanupStack::Pop();



    }
                               
const TUid KUidLocationsIni= { 0x00000001 };



void CWeatherAppUi::LoadLocationsL()
    {

    CDictionaryStore* iniFile = Application()->OpenIniFileLC(CEikonEnv::Static()->FsSession());
    TBool found = iniFile->IsPresentL(KUidLocationsIni);

    iLocationArray = new (ELeave) CArrayPtrFlat<CLocation> (4); 


    if (!found)
        {
        
        RDebug::Print(_L("no locations ini file"));

        CLocation* location = new(ELeave) CLocation(_L("EIDW"), _L("Dublin, IE"));
        CleanupStack::PushL(location);
        iLocationArray->AppendL(location);
        CleanupStack::Pop();

        location = new(ELeave) CLocation(_L("EGPH"), _L("Edinburgh, UK"));
        CleanupStack::PushL(location);
        iLocationArray->AppendL(location);
        CleanupStack::Pop();

        location = new(ELeave) CLocation(_L("EGLL"), _L("London, UK"));
        CleanupStack::PushL(location);
        iLocationArray->AppendL(location);
        CleanupStack::Pop();

        location = new(ELeave) CLocation(_L("KSFO"), _L("San Francisco, US"));
        CleanupStack::PushL(location);
        iLocationArray->AppendL(location);
        CleanupStack::Pop();

        location = new(ELeave) CLocation(_L("EFHF"), _L("Helsinki, FI"));
        CleanupStack::PushL(location);
        iLocationArray->AppendL(location);
        CleanupStack::Pop();


        }
    else
        {
        RDebug::Print(_L("found locations ini file"));

        RDictionaryReadStream readStream;
        readStream.OpenLC (*iniFile, KUidLocationsIni);
        TInt size = readStream.ReadInt16L();
        
        TInt i;
        for (i = 0; i < size; i++)
            {
            CLocation* location = new (ELeave) CLocation();
            CleanupStack::PushL(location);
            location->ConstructFromStreamL(readStream);
            iLocationArray->AppendL(location);
            CleanupStack::Pop();
            }


        CleanupStack::PopAndDestroy(); 

        }


    CleanupStack::PopAndDestroy(); 

    TKeyArrayPtr locNameKey(_FOFF(CLocation,iName), ECmpFolded);
    iLocationArray->Sort(locNameKey);


    RDebug::Print(_L("here"));

    }


const TUid KUidUserSettingsIni= { 0x00000002 };
const TUid KUidUserSettings2Ini= { 0x00000003 };

void CWeatherAppUi::LoadUserSettingsL()
    {

    CDictionaryStore* iniFile = Application()->OpenIniFileLC(CEikonEnv::Static()->FsSession());
    TBool found = iniFile->IsPresentL(KUidUserSettingsIni);

    

    if (!found)
        {
        
        RDebug::Print(_L("no user settings ini file"));

        GLOBALS->iSettings.iTempUnits = ECentigrade;
        GLOBALS->iSettings.iSpeedUnits = EKnots;
        GLOBALS->iSettings.iDistanceUnits = EKilometres;
        GLOBALS->iSettings.iDetailedCloudReports = ECloudDetailed;   
        GLOBALS->iSettings.iUseProxy = EFalse;

        }
    else
        {
        RDebug::Print(_L("found user settings ini file"));

        RDictionaryReadStream readStream;
        readStream.OpenLC (*iniFile, KUidUserSettingsIni);


        GLOBALS->iSettings.iTempUnits = readStream.ReadInt16L();
        GLOBALS->iSettings.iSpeedUnits = readStream.ReadInt16L();
        GLOBALS->iSettings.iDistanceUnits = readStream.ReadInt16L();
        GLOBALS->iSettings.iDetailedCloudReports = readStream.ReadInt16L();   

        GLOBALS->iSettings.iUseProxy = readStream.ReadInt16L();
        GLOBALS->iSettings.iProxyIpAddress.SetAddress(readStream.ReadInt32L());

        iLastLocationIndex  = readStream.ReadInt16L();        

        if ((iLastLocationIndex > -1) && (iLastLocationIndex < iLocationArray->Count()))
            {
            iCurrentLocation = iLocationArray->At(iLastLocationIndex);
            }


        CleanupStack::PopAndDestroy(); 

        }

    // I have a second stream in the ini for settings that i forgot to put in the first one!

    TBool found2 = iniFile->IsPresentL(KUidUserSettings2Ini);
    if (!found2)
        {
        GLOBALS->iSettings.iProxyPort = 80;
        }
    else
        {
        RDictionaryReadStream readStream;
        readStream.OpenLC (*iniFile, KUidUserSettings2Ini);

        GLOBALS->iSettings.iProxyPort = readStream.ReadInt16L();
        RDebug::Print(_L("!! proxy port=%d"), GLOBALS->iSettings.iProxyPort);
        //for future expansion...
        TInt32 dummyInt;
        dummyInt = readStream.ReadInt32L();
        dummyInt = readStream.ReadInt32L();
        dummyInt = readStream.ReadInt32L();

        TBuf<40> dummyString;
        readStream >> dummyString;
        readStream >> dummyString;

        CleanupStack::PopAndDestroy(); 

        }
    

    CleanupStack::PopAndDestroy(); 
    }


void CWeatherAppUi::SaveLocationsL()
    {

    CDictionaryStore* store = Application()->OpenIniFileLC(CEikonEnv::Static()->FsSession());
    RDictionaryWriteStream writeStream;
    writeStream.AssignLC(*store, KUidLocationsIni);


    TInt size = iLocationArray->Count();
    writeStream.WriteInt16L(size);
    TInt i;
    for (i=0; i < size; i++)
        {
        CLocation* location = iLocationArray->At(i);
        location->SaveL(writeStream);
        }

    writeStream.CommitL();
    store->CommitL();


    CleanupStack::PopAndDestroy(); 
    CleanupStack::PopAndDestroy();

    }

void CWeatherAppUi::SaveUserSettingsL()
    {

    CDictionaryStore* store = Application()->OpenIniFileLC(CEikonEnv::Static()->FsSession());



    //stream 1

    RDictionaryWriteStream writeStream;
    writeStream.AssignLC(*store, KUidUserSettingsIni);


    writeStream.WriteInt16L(GLOBALS->iSettings.iTempUnits);
    writeStream.WriteInt16L(GLOBALS->iSettings.iSpeedUnits);
    writeStream.WriteInt16L(GLOBALS->iSettings.iDistanceUnits);
    writeStream.WriteInt16L(GLOBALS->iSettings.iDetailedCloudReports);

    writeStream.WriteInt16L(GLOBALS->iSettings.iUseProxy);
    writeStream.WriteInt32L(GLOBALS->iSettings.iProxyIpAddress.Address());
    writeStream.WriteInt16L(iLastLocationIndex);


    writeStream.CommitL();
    store->CommitL();

    CleanupStack::PopAndDestroy(); 



    //stream 2

    writeStream.AssignLC(*store, KUidUserSettings2Ini);

    writeStream.WriteInt16L(GLOBALS->iSettings.iProxyPort);
    RDebug::Print(_L("!! writing proxy port=%d"), GLOBALS->iSettings.iProxyPort);

    TInt32 dummyInt = 0;
    writeStream.WriteInt32L(dummyInt);
    writeStream.WriteInt32L(dummyInt);
    writeStream.WriteInt32L(dummyInt);

    TBuf<40> dummyString;
    writeStream << dummyString;
    writeStream << dummyString;


    writeStream.CommitL();
    store->CommitL();

    CleanupStack::PopAndDestroy(); 



    

    CleanupStack::PopAndDestroy();

    }


// ----------------------------------------------------
// CWeatherAppUi::~CWeatherAppUi()
// Destructor
// Frees reserved resources
// ----------------------------------------------------
//
CWeatherAppUi::~CWeatherAppUi()
    {

    if (iAppContainer)
        {
        RemoveFromStack( iAppContainer );
        delete iAppContainer;
        }


    iLocationArray->ResetAndDestroy();
    delete iLocationArray;

    if (iTimeLabel) delete iTimeLabel;

    if (iNaviContainer) 
        {
        iNaviContainer->Pop();
        }


    delete iSocketsEngine;
    delete iTxtFormatDelete;
    delete iTxtDeleteLocation;
    delete iTxtModifyLocation;


    delete GLOBALS;
    Dll::SetTls(NULL);


   }

// ------------------------------------------------------------------------------
// CWeatherAppUi::::DynInitMenuPaneL(TInt aResourceId,CEikMenuPane* aMenuPane)
//  This function is called by the EIKON framework just before it displays
//  a menu pane. Its default implementation is empty, and by overriding it,
//  the application can set the state of menu items dynamically according
//  to the state of application data.
// ------------------------------------------------------------------------------
//


void CWeatherAppUi::DynInitMenuPaneL(
    TInt aResourceId,CEikMenuPane* aMenuPane)
    {

    TInt size = iLocationArray->Count();


    TKeyArrayPtr locNameKey(_FOFF(CLocation,iName), ECmpFolded);
    iLocationArray->Sort(locNameKey);

    if ((aResourceId == R_CURRENT_LOCATION_SUBMENU) && (size > 0))
        {
        CEikMenuPaneItem::SData item;
        
        item.iCascadeId = 0;
        item.iFlags = 0;
        
        TInt i;
        for (i = 0; i < size ; i++)
            {
            CLocation* location = iLocationArray->At(i);
            item.iText = location->Name();
            item.iCommandId = 1000 + i;
            aMenuPane->AddMenuItemL(item);
            }


        }

    }


void CWeatherAppUi::EditLocationsCmdL()
    {


    //if there are no locations, there is no point asking
    //if the user wants to Delete or Modify locations, so we
    //don't bother displaying the R_EDIT_LOCATIONS_DIALOG.
    //instead we go straight to the New Location form

    if (iLocationArray->Count() == 0)
        {
        CLocationForm* form = CLocationForm::NewL(NULL, iLocationArray);
        CleanupStack::PushL(form);
        form->ExecuteLD(R_LOCATION_DIALOG);
        CleanupStack::Pop();
        return;
        }


    CAknListQueryDialog* dlg;
    TInt index = 0;
    dlg = new( ELeave ) CAknListQueryDialog( &index );

    if(dlg->ExecuteLD(R_EDIT_LOCATIONS_DIALOG))
        {
        RDebug::Print(_L("%d selected from edit locations"), index);
        
        switch(index)
            {
            case 0 :    //new
                {
                CLocationForm* form = CLocationForm::NewL(NULL, iLocationArray);
                CleanupStack::PushL(form);
                form->ExecuteLD(R_LOCATION_DIALOG);
                CleanupStack::Pop();
                break;
                }
            case 1 :    //modify
                {
                TInt selected;
                if (ShowLocationsPopupListL(*iTxtModifyLocation, selected))
                    {
                    CLocationForm* form = CLocationForm::NewL(iLocationArray->At(selected), iLocationArray);
                    CleanupStack::PushL(form);
                    form->ExecuteLD(R_LOCATION_DIALOG);
                    CleanupStack::Pop();
                    }

                break;
                }
            case 2:     //delete
                {
                TInt selected;
                if (ShowLocationsPopupListL(*iTxtDeleteLocation, selected))
                    {
                    CLocation* location = iLocationArray->At(selected);
                    CAknQueryDialog* dlg = CAknQueryDialog::NewL( CAknQueryDialog::ENoTone );
                    CleanupStack::PushL(dlg);
                    dlg->PrepareLC(R_DELETE_LOCATION_CONFIRMATION_QUERY_DIALOG);
                    TBuf<30> deleteString;
                    deleteString.Format(*iTxtFormatDelete, &(location->Name()));
                    dlg->SetPromptL(deleteString); 
                    if (dlg->RunLD())
                        {
                        //RDebug::Print(_L("delete %d"), selected);
                        iLocationArray->Delete(selected);
                        delete location;
                        }
                    CleanupStack::Pop();
                    }

                break;
                }

            }

        }


    }


TBool CWeatherAppUi::ShowLocationsPopupListL(const TDesC& aTitle, TInt& aSelectedIndex)
    {

    TInt size = iLocationArray->Count();


    __ASSERT_ALWAYS((size > 0),  User::Panic(KEmptyLocations, 0) );

    //only 1 element in list, so no point displaying list
    if (size == 1) 
        {
        aSelectedIndex = 0;
        return ETrue;
        }

    // Create listbox and PUSH it.
    CEikTextListBox* list = new(ELeave) CAknSinglePopupMenuStyleListBox;
    CleanupStack::PushL(list);

    // Create popup list and PUSH it.
    CAknPopupList* popupList = CAknPopupList::NewL(
                                        list, R_AVKON_SOFTKEYS_OK_BACK,
                                        AknPopupLayouts::EMenuWindow);
    CleanupStack::PushL(popupList);

    popupList->SetTitleL(aTitle);

    // initialize listbox.
    list->ConstructL(popupList, CEikListBox::ELeftDownInViewRect);
    list->CreateScrollBarFrameL(ETrue);
    list->ScrollBarFrame()->SetScrollBarVisibilityL(
                                CEikScrollBarFrame::EOff,
                                CEikScrollBarFrame::EAuto);


    CTextListBoxModel* model = list->Model();



    CDesCArrayFlat* items = new (ELeave) CDesCArrayFlat(size);
    CleanupStack::PushL(items);



    TInt i;
    for (i = 0; i < size ; i++)
        {
        CLocation* location = iLocationArray->At(i);
        items->AppendL(location->Name());
        }
    
    model->SetItemTextArray(items);
    model->SetOwnershipType(ELbmDoesNotOwnItemArray);

    TInt popupOk = popupList->ExecuteLD();
    CleanupStack::Pop();    //items
    CleanupStack::Pop();    //popuplist


    aSelectedIndex = list->CurrentItemIndex();

    CleanupStack::PopAndDestroy();  // list
    delete items;

    return popupOk;

    }


void CWeatherAppUi::DisplayLocation()
    {


    iAppContainer->DisplayLocation(iCurrentLocation);
    
    if (!iCurrentLocation) return;

    CEikStatusPane* sp = iEikonEnv->AppUiFactory()->StatusPane();
    
    CAknTitlePane* title = (CAknTitlePane*)sp->ControlL(TUid::Uid(EEikStatusPaneUidTitle));
    title->SetTextL(iCurrentLocation->Name());

    CAknContextPane* cp=(CAknContextPane*)sp->ControlL(TUid::Uid(EEikStatusPaneUidContext));
 
    cp->SetPictureToDefaultL(); // Set application icon.
    sp->DrawNow(); 

    iNaviContainer = (CAknNavigationControlContainer *)sp->ControlL(TUid::Uid(EEikStatusPaneUidNavi)); 
    iNaviContainer->Pop();
    if (iTimeLabel) delete iTimeLabel;
    iTimeLabel = NULL;

    CWeatherInfo* report = iCurrentLocation->Report();

    if (!report)
        return;

    TInt temperature;

    if (report->GetTemperature(temperature))
        {
        CFbsBitmap* bitmap;
        CFbsBitmap* maskBitmap;
        CThermometer::DrawTemperatureL(temperature, bitmap, maskBitmap);
        cp->SetPicture(bitmap, maskBitmap);
        }


    //_LIT(KDateFormat,"%*E%*D%X%*N%*Y %1 %2 '%3 %H:%T");
    _LIT(KDateFormat,"%*E%*D%X%*N %1 %2 %-B%:0%J%:1%T%:3%+B");


    TTime reportTime;
    if (report->GetTime(reportTime))
        {
        //convert to local time
        TLocale locale;
        TTimeIntervalSeconds universalTimeOffset(locale.UniversalTimeOffset());
        reportTime += universalTimeOffset;

        TBuf<40> dateString;
        reportTime.FormatL(dateString, KDateFormat);
        iTimeLabel = iNaviContainer->CreateNavigationLabelL(dateString);
        iNaviContainer->PushL(*iTimeLabel); 

        }


    
    iAppContainer->DrawNow();

    }



_LIT(KWeatherServerName, "weather.noaa.gov");
_LIT8(KWeatherPathFormat, "/pub/data/observations/metar/stations/%S.TXT");
_LIT8(KWeatherPathProxyFormat, "http://weather.noaa.gov/pub/data/observations/metar/stations/%S.TXT");
void CWeatherAppUi::CurrentLocationCmdL(TInt aIndex)
    {
    

    RDebug::Print(_L("current location changed to %d"), aIndex);

    iLastLocationIndex = aIndex;

    CLocation* previousLocation = iCurrentLocation;
    iCurrentLocation = iLocationArray->At(aIndex);

    CAknQueryDialog* dlg = CAknQueryDialog::NewL( CAknQueryDialog::ENoTone );
    CleanupStack::PushL(dlg);
    dlg->PrepareLC(R_RELOAD_LOCATION_QUERY_DIALOG);
    if (dlg->RunLD())
        {


        RDebug::Print(_L("loading %S!\n"), &iCurrentLocation->IcaoCode());

   
        TBuf8<KMaxPathLength> path;
        TBuf8<4> icao;

        CnvUtfConverter::ConvertFromUnicodeToUtf8(icao, iCurrentLocation->IcaoCode());

        RDebug::Print(_L("!! creating wait dialog!!\n"));
        iWaitDialog = new( ELeave )CAknWaitDialog( REINTERPRET_CAST( CEikDialog**, & iWaitDialog ) ); 
        iWaitDialog->SetCallback(this);
        iWaitDialog->ExecuteLD( R_LOADING_REPORT_NOTE );


        if (GLOBALS->iSettings.iUseProxy)
            {
            TBuf<20> buf;
            GLOBALS->iSettings.iProxyIpAddress.Output(buf);
            
            path.AppendFormat(KWeatherPathProxyFormat, &icao);
            iSocketsEngine->StartHttpGetL(buf, path, GLOBALS->iSettings.iProxyPort);
            }
        else
            {
            path.AppendFormat(KWeatherPathFormat, &icao);
            iSocketsEngine->StartHttpGetL(KWeatherServerName, path, 80);
            }


        }
    else if (iCurrentLocation != previousLocation) 
        DisplayLocation();
    
    CleanupStack::Pop();


   

    }

void CWeatherAppUi::SocketStatusNotify(const TDesC& aMessage, TBool aTerminal)
    {


    if (aTerminal) 
        {

        if (iWaitDialog)
            {
            iWaitDialog->ProcessFinishedL(); 
            iWaitDialog = NULL;
            }

        CAknInformationNote* note = new (ELeave)CAknInformationNote();    
        note->ExecuteLD(aMessage);

        }
    else 
        {
        if (iWaitDialog)
            {
            iWaitDialog->SetTextL(aMessage);
            }
        }

    }


void CWeatherAppUi::DialogDismissedL(TInt aButtonId)
    {
    
    RDebug::Print(_L("dismissed"));
    iSocketsEngine->Cancel();


    }

void CWeatherAppUi::SocketStatusNotify(TInt aMsgResourceId, TBool aTerminal)
    {
    TBuf<100> msgBuf; 
    iCoeEnv->ReadResource(msgBuf, aMsgResourceId);
    SocketStatusNotify(msgBuf, aTerminal); 
    }


void CWeatherAppUi::HttpLoadedNotify(TPtrC aBody, TInt aResponseCode)
    {
    //parse report
    

    RDebug::Print(_L("HttpLoadedNotify\n"));
    switch(aResponseCode)
        {
        case 200 : 
            {
            
            HBufC* reportString = aBody.AllocLC();
            iCurrentLocation->ParseReportL(reportString);       //ownership of string transferred
            CleanupStack::Pop();

            //must check dialog is still around as it may have been cancelled
            if (iWaitDialog)
                {
                iWaitDialog->ProcessFinishedL(); 
                iWaitDialog = NULL;
                }
            DisplayLocation();     
            break;
            }
        case 404: SocketStatusNotify(R_TEXT_FILE_NOT_FOUND, ETrue);   break;
        
        default:  
            {
            TBuf<40> msgBuf;
            TBuf<40> msgFmt;
            iCoeEnv->ReadResource(msgFmt, R_TEXT_UNEXPECTED_RESPONSE_CODE);
            msgBuf.Format(msgFmt, aResponseCode);
            SocketStatusNotify(msgBuf, ETrue); 


            }
        }




    }


// ----------------------------------------------------
// CWeatherAppUi::HandleKeyEventL(
//     const TKeyEvent& aKeyEvent,TEventCode /*aType*/)
// ?implementation_description
// ----------------------------------------------------
//
TKeyResponse CWeatherAppUi::HandleKeyEventL(
    const TKeyEvent& /*aKeyEvent*/,TEventCode /*aType*/)
    {
    return EKeyWasNotConsumed;
    }

// ----------------------------------------------------
// CWeatherAppUi::HandleCommandL(TInt aCommand)
// ?implementation_description
// ----------------------------------------------------
//
void CWeatherAppUi::HandleCommandL(TInt aCommand)
    {
    switch ( aCommand )
        {
        case EAknSoftkeyBack:
        case EEikCmdExit:
            {
            SaveLocationsL();
            SaveUserSettingsL();
            Exit();
            break;
            }

        case ECmdUserSettings:
            {
            CUserSettingsDialog* dialog = new(ELeave) CUserSettingsDialog(this);
            dialog->PrepareLC(R_USER_SETTINGS_DIALOG);
            dialog->ConstructL();
            dialog->RunLD();
            break;
            }

        case ECmdEditLocation:
            {
            EditLocationsCmdL();
            break;
            }

        case ECmdAbout:
            {
            
            CAboutDialog* dialog = new(ELeave) CAboutDialog();
	        dialog->SetMopParent(this);
            dialog->ExecuteLD(R_ABOUT_BOX);
            
            /*
            CAknDialog* dialog = new (ELeave)CAknDialog();    
            dialog->SetMopParent(this);
            dialog->ExecuteLD(R_ABOUT_BOX);
            */
            break;
            }

        default:
            {

            if ((aCommand >= 1000) && (aCommand < 1000+iLocationArray->Count()))
                {
                __ASSERT_ALWAYS((iLocationArray->Count() > 0),  User::Panic(KEmptyLocations, 1) );

                CurrentLocationCmdL(aCommand-1000);
                
                                               
                }

            }
            break;      
        }
    }

// End of File  
