/*
 * Copyright Justin Armstrong 2004. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

// INCLUDE FILES
#include "weathercontainer.h"

#include <weather.rsg>
#include "weather.hrh"
#include "weatherappui.h"
#include <eiklabel.h>
#include <gulfont.h>




CWeatherContainer::CWeatherContainer()
    {


    }

// ================= MEMBER FUNCTIONS =======================

// ---------------------------------------------------------
// CWeatherContainer::ConstructL(const TRect& aRect)
// EPOC two phased constructor
// ---------------------------------------------------------
//
void CWeatherContainer::ConstructL(const TRect& aRect)
    {

    CreateWindowL();

    SetRect(aRect);

    iWeatherView = new (ELeave) CWeatherView();

    iWeatherView->SetContainerWindowL(*this);
    iWeatherView->ConstructL(aRect);


    iSBFrame = new (ELeave) CEikScrollBarFrame(this, this, ETrue);
    iSBFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, CEikScrollBarFrame::EOn);
    iSBFrame->SetTypeOfVScrollBar(CEikScrollBarFrame::EArrowHead);

    iYTop = 0;

    ActivateL();

    }

void CWeatherContainer::HandleScrollEventL(CEikScrollBar* /*aScrollBar*/, TEikScrollEvent /*aEventType*/)
    {
    }


void CWeatherContainer::DisplayLocation(CLocation* aLocation)
    {

    
    iWeatherView->DisplayLocation(aLocation);
    
    iYTop = 0;
    iWeatherView->SetPosition(TPoint(0, iYTop));
    UpdateScrollbars();

    }


void CWeatherContainer::UpdateScrollbars()
    {

    TRect rect = iWeatherView->Rect();
    TEikScrollBarModel hSbarModel;
    TEikScrollBarModel vSbarModel;
    TEikScrollBarFrameLayout layout;

    
    if (rect.Height() > Rect().Height()) 
    {

        RDebug::Print(_L("*********** should be on!"));

    
        vSbarModel.iThumbSpan = rect.Height();

        vSbarModel.iScrollSpan = 40;
        vSbarModel.iThumbPosition = -iYTop;


        iSBFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, CEikScrollBarFrame::EOn);
        
        iSBFrame->TileL(&hSbarModel, &vSbarModel, rect, rect, layout);
        iSBFrame->SetVFocusPosToThumbPos(vSbarModel.iThumbPosition);


    
    }
    else {
        RDebug::Print(_L("*********** should be off!"));
        iSBFrame->SetScrollBarVisibilityL(CEikScrollBarFrame::EOff, CEikScrollBarFrame::EOff);

        iSBFrame->TileL(&hSbarModel, &vSbarModel, rect, rect, layout);
    }

    RDebug::Print(_L("iYTop=%d  containerheight=%d  viewheight=%d"), iYTop, Rect().Height(), rect.Height());
    iSBFrame->DrawScrollBarsNow();
    
    ActivateL();

    }


// Destructor
CWeatherContainer::~CWeatherContainer()
    {

    delete iSBFrame;
    delete iWeatherView;

    }

// ---------------------------------------------------------
// CWeatherContainer::SizeChanged()
// Called by framework when the view size is changed
// ---------------------------------------------------------
//
void CWeatherContainer::SizeChanged()
    {
    }

// ---------------------------------------------------------
// CWeatherContainer::CountComponentControls() const
// ---------------------------------------------------------
//
TInt CWeatherContainer::CountComponentControls() const
    {
    return 1;
    }

// ---------------------------------------------------------
// CWeatherContainer::ComponentControl(TInt aIndex) const
// ---------------------------------------------------------
//
CCoeControl* CWeatherContainer::ComponentControl(TInt /*aIndex*/) const
    {

    return iWeatherView;

    }

// ---------------------------------------------------------
// CWeatherContainer::Draw(const TRect& aRect) const
// ---------------------------------------------------------
//
void CWeatherContainer::Draw(const TRect& aRect) const
    {

    CWindowGc& gc = SystemGc();
    gc.SetPenStyle(CGraphicsContext::ENullPen);
    gc.SetBrushColor(KRgbWhite);
    gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
    gc.DrawRect(aRect);

    }

// ---------------------------------------------------------
// CWeatherContainer::HandleControlEventL(
//     CCoeControl* aControl,TCoeEvent aEventType)
// ---------------------------------------------------------
//
void CWeatherContainer::HandleControlEventL(
    CCoeControl* /*aControl*/,TCoeEvent /*aEventType*/)
    {
    }


TKeyResponse CWeatherContainer::OfferKeyEventL(const TKeyEvent& aKeyEvent, TEventCode aType)
    {

    if (aType != EEventKey) return EKeyWasNotConsumed;

    TInt containerHeight = Rect().Height();
    TInt viewHeight = iWeatherView->Rect().Height();

    if (aKeyEvent.iCode==EKeyDownArrow && 
            (viewHeight > containerHeight) &&  (iYTop > (containerHeight - viewHeight)) )
        {
        RDebug::Print(_L("down"));
        iYTop -= 22;
        iWeatherView->SetPosition(TPoint(0, iYTop));
        UpdateScrollbars();
        DrawNow();
        iWeatherView->DrawNow();
        return EKeyWasConsumed;
        }
    if (aKeyEvent.iCode==EKeyUpArrow &&
            (iYTop < 0 ))
        {
        RDebug::Print(_L("up"));
        iYTop += 22;
        iWeatherView->SetPosition(TPoint(0, iYTop));
        UpdateScrollbars();
        DrawNow();
        iWeatherView->DrawNow();
        return EKeyWasConsumed;
        }

    return EKeyWasNotConsumed;
    }


// End of File  
