
/*
 * Copyright Justin Armstrong 2004. 
 *
 * This file is part of the application "Weather"
 * 
 * "Weather" is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Weather is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */


#include "weatherparser.h"
#include "weatherutils.h"
#include <weather.rsg>
#include "weather.hrh"

/*
_LIT(KCond1, "HZ");
_LIT(KCond2, "-RA");
_LIT(KCond3, "+RA");
_LIT(KCond4, "VCRA");
_LIT(KCond5, "BR");
_LIT(KCond6, "FZFG");
_LIT(KCond7, "-BLFG");
_LIT(KCond8, "VCFZDU");
_LIT(KCond9, "-FZDU");
_LIT(KCond10, "-SN");
_LIT(KCond11, "+FU");
_LIT(KCond12, "SHIC");
_LIT(KCond13, "THIC");
_LIT(KCond14, "+SHFU");
_LIT(KCond15, "-FZBR");
_LIT(KCond16, "-SHGS");
*/
/*
_LIT(KCond1, "0HZ");
_LIT(KCond2, "-RAx");
_LIT(KCond3, "+RAxx");
_LIT(KCond4, "VRA");
_LIT(KCond5, "R");
_LIT(KCond6, "RARA");
_LIT(KCond7, "FZFZ");
_LIT(KCond8, "+VCRA");
_LIT(KCond9, "-");
_LIT(KCond10, "VC");
_LIT(KCond11, "+FZ");
_LIT(KCond12, "+SIC");
_LIT(KCond13, "VCFZ");
_LIT(KCond14, "FZ");
_LIT(KCond15, "-SH");
_LIT(KCond16, "");
*/

/*
_LIT(KWind1, "08014KT");
_LIT(KWind2, "28007KT");
_LIT(KWind3, "11011G21KT");
_LIT(KWind4, "00000KT");
_LIT(KWind5, "30005KT");
_LIT(KWind6, "32004KT");
_LIT(KWind7, "17007KT");
_LIT(KWind8, "04008KT");
_LIT(KWind9, "34019G26KT");
_LIT(KWind10, "VRB01KT");
_LIT(KWind11, "00000MPS");
*/
/*
_LIT(KWind1, "8014KT");
_LIT(KWind2, "008014KT");
_LIT(KWind3, "008014G21KT");
_LIT(KWind4, "0KT");
_LIT(KWind5, "008014KP");
_LIT(KWind6, "-008014KT");
_LIT(KWind7, "");
_LIT(KWind8, "KT");
_LIT(KWind9, "08014GKT");
_LIT(KWind10, "VRBKT");
_LIT(KWind11, "MPS");
*/



/*
_LIT(KCloud1, "FEW022");
_LIT(KCloud2, "SCT100");
_LIT(KCloud3, "SCT013");
_LIT(KCloud4, "BKN018");
_LIT(KCloud5, "FEW025");
_LIT(KCloud6, "SCT060");
_LIT(KCloud7, "FEW000");
_LIT(KCloud8, "CLR");
_LIT(KCloud9, "CAVOK");
*/
/*
_LIT(KCloud1, "XXX022");
_LIT(KCloud2, "SC100");
_LIT(KCloud3, "SC0133");
_LIT(KCloud4, "BKN01");
_LIT(KCloud5, "FW025");
_LIT(KCloud6, "SCRT060");
_LIT(KCloud7, "FEW0020");
_LIT(KCloud8, "CLXFEW");
_LIT(KCloud9, "");
*/

/*
_LIT(KTemp1, "04/M00");
_LIT(KTemp2, "01/M01");
_LIT(KTemp3, "05/04");
_LIT(KTemp4, "01///");
_LIT(KTemp5, "03/M02");
_LIT(KTemp6, "M13/M19");
*/
/*
_LIT(KTemp1, "0/M00");
_LIT(KTemp2, "/1");
_LIT(KTemp3, "1/");
_LIT(KTemp4, "/");
_LIT(KTemp5, "03/M02/");
_LIT(KTemp6, "");
_LIT(KTemp7, "M132/M19");
_LIT(KTemp8, "M13/M192");
*/


//--------------------------------------------

_LIT(KEmpty, "");

_LIT(KNoReportSet, "CWeatherParser: no report set");


_LIT(KDateMatch, "\?\?\?\?/\?\?/\?\?");
_LIT(KTimeMatch, "\?\?:\?\?");


TBool CWeatherParser::ParseDateAndTime(TDesC& aDateTimeParam)
    {

    __ASSERT_ALWAYS(iWeatherReport,  User::Panic(KNoReportSet, 0) );

    TLex dtLexer(aDateTimeParam);

    dtLexer.Mark();
    dtLexer.SkipCharacters();

    TPtrC dateStr = dtLexer.MarkedToken();

    if ((dateStr.MatchC(KDateMatch())) == KErrNotFound)
        {
        //RDebug::Print(_L("dateStr"));
        return EFalse;
        }

    if (dtLexer.Eos()) return EFalse;

    dtLexer.SkipSpaceAndMark();
    dtLexer.SkipCharacters();

    TPtrC timeStr = dtLexer.MarkedToken();

    if ((timeStr.MatchC(KTimeMatch())) == KErrNotFound)
        {
        //RDebug::Print(_L("timeStr <%S>"), &timeStr);
        return EFalse;
        }
    

    TInt year = 0;
        {
        TPtrC yearStr = dateStr.Mid(0, 4);
        TLex lexer(yearStr);
        if (lexer.Val(year) != KErrNone)
            {
            //RDebug::Print(_L("1"));
            return EFalse;
            }

        }

    TInt month = 1;
        {
        TPtrC monthStr = dateStr.Mid(5, 2);
        TLex lexer(monthStr);
        if (lexer.Val(month) != KErrNone)
            {
            //RDebug::Print(_L("2"));
            return EFalse;
            }

        }

    TInt day = 1;
        {
        TPtrC dayStr = dateStr.Mid(8, 2);
        TLex lexer(dayStr);
        if (lexer.Val(day) != KErrNone)
            {
            //RDebug::Print(_L("3"));
            return EFalse;
            }

        }



    TInt hour = 0;
        {
        TPtrC hourStr = timeStr.Mid(0, 2);
        TLex lexer(hourStr);
        if (lexer.Val(hour) != KErrNone)
            {
            //RDebug::Print(_L("4"));
            return EFalse;
            }

        }

    TInt minute = 0;
        {
        TPtrC minuteStr = timeStr.Mid(3, 2);
        TLex lexer(minuteStr);
        if (lexer.Val(minute) != KErrNone)
            {
            //RDebug::Print(_L("5"));
            return EFalse;
            }

        }

    //make a string in the format TTime requires
    //note: symbian months and days start at 0!
    TBuf<20> timeString;    
    timeString.Format(_L("%04d%02d%02d:%02d%02d00"), year, month-1, day-1, hour, minute);
    RDebug::Print(_L("making <%S> into a TTime"), &timeString);



    TTime time;
    if (time.Set(timeString) == KErrGeneral)
        return EFalse;

    iWeatherReport->SetTime(time);     

    return ETrue;
    }

//////////////////////////////////////////////////////////////////////////

_LIT(VC, "VC");
TBool CWeatherParser::ParseCondition(TDesC& aConditionString)
    {

    __ASSERT_ALWAYS(iWeatherReport,  User::Panic(KNoReportSet, 1) );

    TInt length = aConditionString.Length();
    if (length < 2)
        return EFalse;

    TIntensity intensity = EModerate;

    TInt pos = 0;

    if (length > 2)
        {

        if (aConditionString[0] == '-')
            {
            pos++;
            intensity = ELight;
            }
        else if (aConditionString[0] == '+')
            {
            pos++;
            intensity = EHeavy;
            }
        else if (aConditionString.Mid(0,2).Compare(VC) == 0)
            {
            pos += 2;
            intensity = EVicinity;
            }
        }



    TPhenomenonModifier modifier = ENoMod;

    if ((length-pos != 2) && (length-pos != 4))
        return EFalse;

    if (length-pos == 4)  //we should have a modifier
        {

        TPtrC modifierCode = aConditionString.Mid(pos, 2);
        modifier = (TPhenomenonModifier)iModifierDict->GetIntForString(modifierCode);

        if (modifier == ETupleNotFound) return EFalse;
        pos += 2;

        }

    TPtrC phenomenonCode = aConditionString.Mid(pos, 2);
    TPhenomenon phenomenon = (TPhenomenon)iPhenomenonDict->GetIntForString(phenomenonCode);

    if (phenomenon == ETupleNotFound) return EFalse;


    iWeatherReport->AddConditionL(intensity, modifier, phenomenon);
   
    return ETrue;

    }

////////////////////////////////////////////////////////////////////////////////

_LIT(KPressureMatchHg, "A\?\?\?\?");    //inches of mercury
_LIT(KPressureMatchMb, "Q\?\?\?\?");    //millibars / hectopascals

TBool CWeatherParser::ParsePressure(TDesC& aPressureParam)
    {

    TBool millibars;

    if ( (aPressureParam.Match(KPressureMatchHg())) != KErrNotFound)
        {
        millibars = EFalse;
        }
    else if ((aPressureParam.Match(KPressureMatchMb())) != KErrNotFound)
        {
        millibars = ETrue;
        }
    else return EFalse;

    TInt value;
    TPtrC valueStr = aPressureParam.Mid(1,4);
    TLex lexer(valueStr);
    if (lexer.Val(value) != KErrNone)
        {
        return EFalse;
        }

    if (!millibars)
        {
        iWeatherReport->SetPressure((TInt)(value * (33.8639/100)));
        }
    else iWeatherReport->SetPressure(value);

    return ETrue;
    }


////////////////////////////////////////////////////////////////////////////////


_LIT(KTempMatchPP, "\?\?/\?\?");    //??/?? e.g. 23/04
_LIT(KTempMatchMP, "M\?\?/\?\?");   //M??/?? e.g. M02/01
_LIT(KTempMatchPM, "\?\?/M\?\?");   //??/M?? e.g. 02/M01
_LIT(KTempMatchMM, "M\?\?/M\?\?");  //M??/M?? e.g. M02/M01
TBool CWeatherParser::ParseTemp(TDesC& aTempParam)
    {

    __ASSERT_ALWAYS(iWeatherReport,  User::Panic(KNoReportSet, 2) );

    TInt minusTemp;
    TInt minusDew;
    TInt tempPos;
    TInt dewPos;

    if ( (aTempParam.Match(KTempMatchPP())) != KErrNotFound)
        {
        minusTemp = EFalse;
        tempPos = 0;
        minusDew = EFalse;
        dewPos = 3;
        }
    else if ((aTempParam.Match(KTempMatchMP())) != KErrNotFound)
        {
        minusTemp = ETrue;
        tempPos = 1;
        minusDew = EFalse;
        dewPos = 4;
        }
    else if ((aTempParam.Match(KTempMatchPM())) != KErrNotFound)
        {
        minusTemp = EFalse;
        tempPos = 0;
        minusDew = ETrue;
        dewPos = 4;
        }
    else if ((aTempParam.Match(KTempMatchMM())) != KErrNotFound)
        {
        minusTemp = ETrue;
        tempPos = 1;
        minusDew = ETrue;
        dewPos = 5;
        }
    else return EFalse;


    TPtrC tempStr = aTempParam.Mid(tempPos, 2);

    
    TInt temperature = 0;
        {

        TLex lexer(tempStr);
        if (lexer.Val(temperature) != KErrNone)
            {
            //RDebug::Print(_L("problem with temperature"));
            return EFalse;
            }

        }

    if (minusTemp) temperature = -temperature;

    //RDebug::Print(_L("temperature is %d"), temperature);
    iWeatherReport->SetTemperature(temperature);

    TPtrC dewStr = aTempParam.Mid(dewPos, 2);

    //dew point is optional
    TInt dewTemp = 0;
        {

        TLex lexer(dewStr);
        if (lexer.Val(dewTemp) == KErrNone)
            {
            if (minusDew) dewTemp = -dewTemp;
            iWeatherReport->SetDewPoint(dewTemp);
            }
            
        }



    return ETrue;

    }

///////////////////////////////////////////////////////////////////////////

_LIT(KCumulonimbusMatch, "\?\?\?\?\?\?CB");
_LIT(KToweringCumulusMatch, "\?\?\?\?\?\?TCU");

TBool CWeatherParser::ParseCloudCover(TDesC& aCloudString)
    {

    __ASSERT_ALWAYS(iWeatherReport,  User::Panic(KNoReportSet, 3) );

    //check if we match CLR or SKC or CAVOK on its own (no clouds)
    TCloudCoverage coverage = (TCloudCoverage)iCloudDict->GetIntForString(aCloudString);
    if (coverage != ETupleNotFound) 
        {
        iWeatherReport->AddCloudL(coverage, 0);
        return ETrue;
        }

    //otherwise assume its something like "SCT020"

    //TODO - actually store these attributes somewhere
    if ((aCloudString.Match(KCumulonimbusMatch())) != KErrNotFound)
        {
        RDebug::Print(_L("Cumulonimbus!!"));
        }
    else if ((aCloudString.Match(KToweringCumulusMatch())) != KErrNotFound)
        {
        RDebug::Print(_L("towering cumulus!!"));
        }
    else if (aCloudString.Length() != 6)
        return EFalse;

    TInt height;
        {
        TPtrC heightStr = aCloudString.Mid(3,3);
        TLex lexer(heightStr);
        if (lexer.Val(height) != KErrNone)
            {
            //RDebug::Print(_L("problem with cloud height"));
            return EFalse;
            }

        height = height*100;
        }

    TPtrC cloudCode = aCloudString.Mid(0,3);

    coverage = (TCloudCoverage)iCloudDict->GetIntForString(cloudCode);
    if (coverage == ETupleNotFound) return EFalse;


    iWeatherReport->AddCloudL(coverage, height);


    return ETrue;

    }



///////////////////////////////////////////////////////////////////////

_LIT(KDirectionOutOfRange, "direction out of range");

TCompassDirection CWeatherParser::CompassDirection(TInt degrees)
    {

    if (degrees >= 0 && degrees <= 10) return ECompassN;
    else if (degrees >= 11 && degrees <= 34) return ECompassNNE;
    else if (degrees >= 35 && degrees <= 55) return ECompassNE;
    else if (degrees >= 56 && degrees <= 79) return ECompassENE;
    else if (degrees >= 80 && degrees <= 100) return ECompassE;
    else if (degrees >= 101 && degrees <= 124) return ECompassESE;
    else if (degrees >= 125 && degrees <= 145) return ECompassSE;
    else if (degrees >= 146 && degrees <= 169) return ECompassSSE;
    else if (degrees >= 170 && degrees <= 190) return ECompassS;
    else if (degrees >= 191 && degrees <= 214) return ECompassSSW;
    else if (degrees >= 215 && degrees <= 235) return ECompassSW;
    else if (degrees >= 236 && degrees <= 259) return ECompassWSW;
    else if (degrees >= 260 && degrees <= 280) return ECompassW;
    else if (degrees >= 281 && degrees <= 304) return ECompassWNW;
    else if (degrees >= 305 && degrees <= 325) return ECompassNW;
    else if (degrees >= 326 && degrees <= 349) return ECompassNNW;
    else if (degrees >= 350 && degrees <= 360) return ECompassN;

    __ASSERT_ALWAYS(0, User::Panic(KDirectionOutOfRange, 0));
    return ECompassError;        

	}



///////////////////////////////////////////////////////////////////////


_LIT(KVisibilitySMMatch, "*SM");     //TODO - fractions
_LIT(KVisibilityMetresMatch, "\?\?\?\?");

TBool CWeatherParser::ParseVisibilty(TDesC& aVisParam)
    {
    RDebug::Print(_L("ParseVisibilty <%S>"), &aVisParam);
    if (aVisParam.Match(KVisibilitySMMatch()) != KErrNotFound)
        {
        TInt miles;
        TPtrC milesStr = aVisParam.Mid(0,aVisParam.Length()-2);
        RDebug::Print(_L("milesStr <%S>"), &milesStr);
        TLex lexer(milesStr);
        if (lexer.Val(miles) != KErrNone)
            {
            return EFalse;
            }
        RDebug::Print(_L("visibility %d  (%d metres)"), miles, (TInt)MilesToMetres(miles));
        iWeatherReport->SetVisibility((TInt)MilesToMetres(miles));
        return ETrue;
        }
    else if (aVisParam.Match(KVisibilityMetresMatch()) != KErrNotFound)
        {
        TInt metres;
        TLex lexer(aVisParam);
        if (lexer.Val(metres) != KErrNone)
            {
            return EFalse;
            }
        RDebug::Print(_L("visibility %d"), metres);
        iWeatherReport->SetVisibility(metres);
        return ETrue;
        }
    
    else return EFalse;

    }


///////////////////////////////////////////////////////////////////////
//these are not that common and we ignore them anyway!

_LIT(KModAuto, "AUTO");
_LIT(KModCorrected, "COR");

TBool CWeatherParser::ParseModifier(TDesC& aModParam)
    {

     if ((aModParam.CompareC(KModAuto) == 0) ||
        (aModParam.CompareC(KModCorrected) == 0))
        {
        return ETrue;
        }

    return EFalse;

    }

///////////////////////////////////////////////////////////////////////




//if only symbian had some proper regexps!

_LIT(KWindKnotsMatch, "\?\?\?\?\?KT");
_LIT(KWindKnotsGMatch, "\?\?\?\?\?G\?\?KT");
_LIT(KWindMPSMatch, "\?\?\?\?\?MPS");
_LIT(KWindMPSGMatch, "\?\?\?\?\?G\?\?MPS");
_LIT(KWindKMHMatch, "\?\?\?\?\?KMH");
_LIT(KWindKMHGMatch, "\?\?\?\?\?G\?\?KMH");
_LIT(KVrb, "VRB");
_LIT(KCalm, "00000");


TBool CWeatherParser::ParseWind(TDesC& aWindParam)
    {

    __ASSERT_ALWAYS(iWeatherReport,  User::Panic(KNoReportSet, 4) );


    if (aWindParam.Length() < 7)
        return EFalse;

    TBool directionVariable = EFalse;

    TSpeedUnit speedUnit;

    if ((aWindParam.Match(KWindKnotsMatch()) != KErrNotFound) ||
        (aWindParam.Match(KWindKnotsGMatch()) != KErrNotFound))
        {
        speedUnit = EKnots;
        }
    else if ((aWindParam.Match(KWindMPSMatch()) != KErrNotFound) ||
            (aWindParam.Match(KWindMPSGMatch()) != KErrNotFound))
        {
        speedUnit = EMps;
        }
    else if ((aWindParam.Match(KWindKMHMatch()) != KErrNotFound) ||
            (aWindParam.Match(KWindKMHGMatch()) != KErrNotFound))
        {
        speedUnit = EKph;
        }
    else return EFalse; //can't be a wind parameter


        //quick test to see if calm
        {
        TPtrC calmStr = aWindParam.Mid(0,5);
        if (calmStr.Compare(KCalm) == 0)
            {
            //RDebug::Print(_L("Calm"));
            iWeatherReport->SetWind(0, ECompassError, 0, EFalse);
            return ETrue;
            }

        }

    //HBufC* compassDirection = NULL;

    TCompassDirection compassDirection = ECompassError;

    TInt pos = 0;

        {

        TInt degrees = 0; //0-359

        TPtrC directionStr = aWindParam.Mid(pos,3);

        if (directionStr.CompareC(KVrb) == 0)   //"VRB"
            {
            //compassDirection = iTxtDirectionVaries;
            directionVariable = ETrue;
            }
        else
            {

            TLex lexer(directionStr);
            if (lexer.Val(degrees) == KErrNone)
                {
                    if ((degrees < 0) || (degrees > 360)) return EFalse;

                    compassDirection = CompassDirection(degrees);
                    //if (!compassDirection) return EFalse;
                    //RDebug::Print(_L("%d -> %S"), degrees, compassDirection);
                }
            else return EFalse;
                
            }

        }
        pos += 3;

    TInt speed = 0;
 
        {
        TInt units;
        TPtrC speedStr = aWindParam.Mid(pos,2);
        TLex lexer(speedStr);
        if (lexer.Val(units) != KErrNone)
            {
            //RDebug::Print(_L("problem with speed"));
            return EFalse;
            }

        if (speedUnit == EKph)
            {
            speed = (TInt)KphToKnots(units);
            }
        else if (speedUnit == EMps)
            {
            speed = (TInt)MpsToKnots(units);
            }
        else
            {
            speed = units;  //knots
            }

        }
        pos += 2;
  

    TInt gustSpeed = 0;

    //gust winds are optional
    if (aWindParam[pos] == 'G')
        {
        pos++;
        TInt units = 0;
        TPtrC gustStr = aWindParam.Mid(pos, 2);
        TLex lexer(gustStr);
        if (lexer.Val(units) != KErrNone)
            {
            //RDebug::Print(_L("problem with gust"));
            return EFalse;
            }

        if (speedUnit == EKph)
            {
            gustSpeed = (TInt)KphToKnots(units);
            }
        else if (speedUnit == EMps)
            {
            gustSpeed = (TInt)MpsToKnots(units);
            }
        else
            {
            gustSpeed = units;  //knots
            }

   
        }
   iWeatherReport->SetWind(speed, compassDirection, gustSpeed, directionVariable);

    return ETrue;

    }

CWeatherParser::CWeatherParser()
    {

    }

CWeatherParser::~CWeatherParser()
    {
    delete iPhenomenonDict;
    delete iModifierDict;
    delete iCloudDict;
    }



_LIT(KRmk, "RMK");
CWeatherInfo* CWeatherParser::ParseReportL(TDesC& aMetarData)
    {

   
    RDebug::Print(_L("\nreport=<%S>"), &aMetarData);

    TLex lexer(aMetarData);

    lexer.SkipSpaceAndMark();
    lexer.SkipCharacters();
    lexer.SkipSpace();
    lexer.SkipCharacters();

    RDebug::Print(_L("first param <%S>"), &(lexer.MarkedToken()));

    CWeatherInfo* report = CWeatherInfo::NewLC();
    iWeatherReport = report;

    TTime reportTime;
    if (!ParseDateAndTime(lexer.MarkedToken()))
        {
        RDebug::Print(_L("could not parse time, parse aborted"));

        CleanupStack::Pop();
        iWeatherReport = NULL;
        return report;
        }



    //skip ICAO code
    lexer.SkipSpace();
    lexer.SkipCharacters();

    //skip zulu time
    lexer.SkipSpace();
    lexer.SkipCharacters();

    lexer.SkipSpaceAndMark();
    lexer.SkipCharacters();

    if (ParseModifier(lexer.MarkedToken()))
        {
        lexer.SkipSpaceAndMark();
        lexer.SkipCharacters();
        }

    ParseWind(lexer.MarkedToken());

    lexer.SkipSpaceAndMark();
    lexer.SkipCharacters();

    if (ParseVisibilty(lexer.MarkedToken()))
        {
        lexer.SkipSpaceAndMark();
        lexer.SkipCharacters();
        }

    while(!lexer.Eos())
        {
        RDebug::Print(_L("-%S-\n"), &lexer.MarkedToken());

        if (lexer.MarkedToken().Compare(KRmk) == 0) break;

        //RDebug::Print(_L("param <%S>"), &(lexer.MarkedToken()));
        if (  ParseCondition(lexer.MarkedToken()) ||
              ParseCloudCover(lexer.MarkedToken()) ||
              ParseTemp(lexer.MarkedToken()) ||
              ParsePressure(lexer.MarkedToken()) ) {}
            //RDebug::Print(_L("param <%S> unknown"), &(lexer.MarkedToken()));

        lexer.SkipSpaceAndMark();
        lexer.SkipCharacters();
        }
        
    CleanupStack::Pop();
    iWeatherReport = NULL;
    return report;

    }






void CWeatherParser::ConstructL(CCoeEnv* aCoeEnv)
    {

    TResourceReader reader;

    aCoeEnv->CreateResourceReaderLC(reader, R_METAR_CODE_PHENOMENA);
    iPhenomenonDict = CStringIntDict::NewLC(reader);
    CleanupStack::Pop();    //iPhenomenonDict
    CleanupStack::PopAndDestroy();    //reader 


    aCoeEnv->CreateResourceReaderLC(reader, R_METAR_CODE_CLOUDS);
    iCloudDict = CStringIntDict::NewLC(reader);
    CleanupStack::Pop();    //iPhenomenonDict
    CleanupStack::PopAndDestroy();    //reader 

    aCoeEnv->CreateResourceReaderLC(reader, R_METAR_CODE_MODIFIERS);
    iModifierDict = CStringIntDict::NewLC(reader);
    CleanupStack::Pop();    //iPhenomenonDict
    CleanupStack::PopAndDestroy();    //reader 

    /*
    ParseCondition(KCond1());
    ParseCondition(KCond2());
    ParseCondition(KCond3());
    ParseCondition(KCond4());
    ParseCondition(KCond5());
    ParseCondition(KCond7());
    ParseCondition(KCond8());
    ParseCondition(KCond6());
    ParseCondition(KCond9());
    ParseCondition(KCond10());
    ParseCondition(KCond11());
    ParseCondition(KCond12());
    ParseCondition(KCond13());
    ParseCondition(KCond14());
    ParseCondition(KCond15());
    ParseCondition(KCond16());

*/
/*
    ParseWind(KWind1());
    ParseWind(KWind2());
    ParseWind(KWind3());
    ParseWind(KWind4());
    ParseWind(KWind5());
    ParseWind(KWind6());
    ParseWind(KWind7());
    ParseWind(KWind8());
    ParseWind(KWind9());
    ParseWind(KWind10());
    ParseWind(KWind11());
*/
/*
    ParseCloudCover(KCloud1());
    ParseCloudCover(KCloud2());
    ParseCloudCover(KCloud3());
    ParseCloudCover(KCloud4());
    ParseCloudCover(KCloud5());
    ParseCloudCover(KCloud6());
    ParseCloudCover(KCloud7());
    ParseCloudCover(KCloud8());
    ParseCloudCover(KCloud9());
*/
/*
    ParseTemp(KTemp1());
    ParseTemp(KTemp2());
    ParseTemp(KTemp3());
    ParseTemp(KTemp4());
    ParseTemp(KTemp5());
    ParseTemp(KTemp6());
    ParseTemp(KTemp7());
    ParseTemp(KTemp8());
*/
    }




CWeatherParser* CWeatherParser::NewLC(CCoeEnv* aCoeEnv)
	{
	CWeatherParser* self = new (ELeave) CWeatherParser();
	CleanupStack::PushL(self);
	self->ConstructL(aCoeEnv);

	return self;
	}


